use std::{
    path::{Path, PathBuf},
    time::{Duration, Instant},
};

use ignore::{
    overrides::{Override, OverrideBuilder},
    WalkBuilder,
};

fn main() {
    // Update path to point to a large directory structure containing (git)ignore files,
    // e.g. a projects directory that can be traversed within some seconds.
    let path = PathBuf::from("/");
    let timeout = Duration::from_secs(600);
    let overrides = Walk::overrides();
    let start = Instant::now();
    loop {
        if start.elapsed() >= timeout {
            println!("timeout");
            break;
        }
        println!("walk");
        // In a real app, we want to customize paths for each walk,
        // and sometimes provide new overrides.
        let walk = Walk {};
        walk.walk(&path, overrides.clone());
        println!("walk complete");
    }
}

struct Walk {}

impl Walk {
    fn walk(&self, path: &Path, overrides: Override) {
        let mut walk_builder = WalkBuilder::new(path);
        // *No* unbounded memory growth when setting the following ignore flags to false...
        walk_builder
            // Adding overrides leads to faster unbounded growth when ignore flags are true.
            .overrides(overrides)
            .ignore(true)
            .parents(true)
            .git_ignore(true)
            .git_global(true)
            .git_exclude(true);
        let walk = walk_builder.threads(6).build_parallel();
        walk.run(|| {
            Box::new(move |_result| {
                use ignore::WalkState::*;
                Continue
            })
        });
    }

    #[allow(dead_code)]
    fn overrides() -> Override {
        let exclude_patterns: Vec<&str> = vec![
            "!**/$RECYCLE.BIN",
            "!**/*.app/**",
            "!**/*.appex/**",
            "!**/*.bundle/**",
            "!**/*.dSYM/**",
            "!**/*.flplugin/**",
            "!**/*.framework/**",
            "!**/*.icdplugin/**",
            "!**/*.ideplugin/**",
            "!**/*.key/**",
            "!**/*.lproj/**",
            "!**/*.lrcat/**",
            "!**/*.lrcat-data/**",
            "!**/*.lrdata/**",
            "!**/*.lrlibrary/**",
            "!**/*.musiclibrary/**",
            "!**/*.numbers/**",
            "!**/*.pages/**",
            "!**/*.photoslibrary/**",
            "!**/*.playground/**",
            "!**/*.plugin/**",
            "!**/*.pkg/**",
            "!**/*.xcassets/**",
            "!**/*.xcodeproj/**",
            "!**/*.xcworkspace/**",
            "!*.temp",
            "!*.tmp",
            "!node_modules",
        ];
        let mut override_builder = OverrideBuilder::new("/");
        for glob in &exclude_patterns {
            override_builder.add(glob).unwrap();
        }
        override_builder.build().unwrap()
    }
}
